#ifndef ddh_undoable_hide_H
#define ddh_undoable_hide_H

#include "ac_plugin.h"
#include "Undoable.h"

// Additional declares
extern "C"
{
Prototype List *ac_selection_get_part_selected_objects();
Prototype List *get_unselected_objects();
}

//---------------------------------------------------------------------------------------
// Base code courtesy of Andy Colebourne
class DDHUndoableHide : public Undoable
{
protected:

   List *obs;
   ACSelection *selection_snapshot;
   bool mshow;

public:

   DDHUndoableHide(List *oblist, bool show_ind)
   {
      obs = list_clone(oblist);
      mshow = show_ind;
      if( mshow )
      {
         selection_snapshot = NULL;
      }
      else
      {
         // Hiding objects - snapshot all selected objects in case we hide some selected objects
         selection_snapshot = ac_selection_get_snapshot();   
      }
   }

   void execute()
   {
      int c = 0;
      for (List *p = obs; p != NULL; p = p->next)
      {
         c++;
         object_set_visible( (ACObject *)p->data, mshow);
         if( !mshow )
         {
            ac_unselect_object( (ACObject *)p->data );
         }
      }
      redraw_all();
      display_status();
   }

   void undo()
   {
      for (List *p = obs; p != NULL; p = p->next)
         object_set_visible( (ACObject *)p->data, !mshow);
      if( selection_snapshot != NULL )
      {
         ac_selection_restore_from_snapshot(selection_snapshot);
      }
      redraw_all();
      display_status();
   }

   ~DDHUndoableHide()
   {
      if( selection_snapshot != NULL )
      {
         delete selection_snapshot;
      }
      list_free(&obs);
   }
};
//---------------------------------------------------------------------------------------

#endif

