#include "cddhlayer.h"
#include "DDHUndoableHide.h"
#include <string.h>

// Static/singleton inits
bool CDDHLayer::mInstanceInd = false;
CDDHLayer *CDDHLayer::mSingle = NULL;

//---------------------------------------------------------------------------------------
CDDHLayer *CDDHLayer::getSingleton()
{
   if( !mInstanceInd ) {
      mSingle = new CDDHLayer();
      mInstanceInd = true;
   }
   return mSingle;
}
//---------------------------------------------------------------------------------------
CDDHLayer::CDDHLayer()
{
   mSaveLayerInfo = false;
   printf( "Creating layer object\n" );
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      mHead[ix] = NULL;
      mTail[ix] = NULL;
   }
}
//---------------------------------------------------------------------------------------
CDDHLayer::~CDDHLayer()
{
   printf( "Deleting layer object\n" );
   mInstanceInd = false;
   clear();
}
//---------------------------------------------------------------------------------------
void CDDHLayer::clear()
{
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      while( mHead[ix] != NULL )
      {
         mTail[ix] = mHead[ix]->next;
         delete mHead[ix];
         mHead[ix] = mTail[ix];
      }
      mTail[ix] = NULL;
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::updateObjDataInfo( ACObject *o, int lno )
{
   if( !object_exists( o ) || ac_object_get_type( o ) == OBJECT_WORLD )
   {
      return;
   }
   // Convert the layer number to a character starting at the ascii zero value
   lno += DDHLAYER_STARTVAL;
   // Get the object data
   char *odata = ac_object_get_data( o );
   if( odata == NULL )
   {
      // No object data - add new data
      char dline[32];
      sprintf( dline, "%s%c", "//dlayer", ( char )lno );
      object_set_userdata( o, dline );
      return;
   }
   char *p = strstr( odata, "//dlayer" );
   // Determine whether the ddhlayer indicator already exists
   if( p != NULL )
   {
      p += 8;
      if( *p == '\0' )
      {
         // We've hit the NULL terminator - append the string
         char *dline = new char[strlen( odata ) + 2];
         sprintf( dline, "%s%c", odata, ( char )lno );
         object_set_userdata( o, dline );
         delete[] dline;
      }
      else
      {
         *p = (( char )lno );
      }
   }
   else
   {
      char *dline = new char[strlen( odata ) + 13];
      sprintf( dline, "%s%c%s%c", odata, 10, "//dlayer", ( char )lno );
      object_set_userdata( o, dline );
      delete[] dline;
   }
}
//---------------------------------------------------------------------------------------
CDDHData *CDDHLayer::findObjInLayer( int lno, void *o )
{
   for( CDDHData *d = mHead[lno]; d != NULL; d = d->next )
   {
      if( d->data == o )
      {
         return d;
      }
   }
   return NULL;
}
//---------------------------------------------------------------------------------------
int CDDHLayer::findObjInAllLayers( void *o )
{
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      if( findObjInLayer( ix, o ) != NULL )
      {
         return ix;
      }
   }
   return -1;
}
//---------------------------------------------------------------------------------------
void CDDHLayer::removeObjFromLayer( int lno, CDDHData *d )
{
   if( d != NULL )
   {
      if( d->prev != NULL )
      {
         d->prev->next = d->next;
      }
      if( d->next != NULL )
      {
         d->next->prev = d->prev;
      }
      if( mHead[lno] == d )
      {
         mHead[lno] = mHead[lno]->next;
      }
      if( mTail[lno] == d )
      {
         mTail[lno] = mTail[lno]->prev;
      }
      delete d;
   }
}
//---------------------------------------------------------------------------------------
int CDDHLayer::getLayerFromObjData( ACObject *o )
{
   char *odata = ac_object_get_data( o );
   if( odata == NULL )
   {
      return -1;
   }
   char *p = strstr( odata, "//dlayer" );
   if( p != NULL )
   {
      p += 8;
      if( *p != '\0' )
      {
         return (( int )( *p )) - DDHLAYER_STARTVAL;
      }
   }
   return -1;
}
//---------------------------------------------------------------------------------------
void CDDHLayer::moveObjToLayer( int lno, void *o )
{
   // Remove the old object reference
   bool addObject = true;
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      CDDHData *obj = findObjInLayer( ix, o );
      if( obj != NULL )
      {
         // If it's our target layer, our work here is done
         if( lno == ix )
         {
            addObject = false;
         }
         // Otherwise, remove the old reference
         else
         {
            removeObjFromLayer( ix, obj );
         }
         break;
      }
   }
   // Add the item
   if( addObject )
   {
      if( mHead[lno] == NULL )
      {
         mHead[lno] = new CDDHData();
         mTail[lno] = mHead[lno];
      }
      else
      {
         mTail[lno]->next = new CDDHData();
         mTail[lno]->next->prev = mTail[lno];
         mTail[lno] = mTail[lno]->next;
      }
      // Set the data value
      mTail[lno]->data = o;
      //printf( "Added object to layer %d\n", lno );
   }
   // Update the data field
   if( mSaveLayerInfo )
   {
      updateObjDataInfo(( ACObject * )o, lno );
      //printf( "Updated object data in layer %d\n", lno );
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::moveSelectionRecurse( List *objs, int lno, int &objsmoved )
{
   // Loop through selection
   for( List *po = objs; po != NULL; po = po->next )
   {
      // Ensure item doesn't already exist in this
      ACObject *o = ( ACObject * )po->data;
      moveObjToLayer( lno, o );
      ++objsmoved;
      // Add children
      List *chlist = ac_object_get_childrenlist( o );
      if( chlist != NULL )
      {
         moveSelectionRecurse( ac_object_get_childrenlist( o ), lno, objsmoved );
      }
   }
   // Get child objects
}
//---------------------------------------------------------------------------------------
void CDDHLayer::moveSelection( int lno )
{
   // Exit if not in object/group mode
   int smode = ac_get_select_mode();
   if( smode != SELECT_GROUP && smode != SELECT_OBJECT )
   {
      message_dialog( "layer : cannot add objects to layer while not in object/group mode" );
      return;
   }
   if( lno < 0 || lno >= DDHLAYER_COUNT )
   {
      lno = 0;
   }
   // Add objects
   int objsmoved = 0;
   List *oblist = ac_selection_get_objects();
   moveSelectionRecurse( oblist, lno, objsmoved );
   list_free( &oblist );
   display_message( "layer : moved %d objects to layer %d", objsmoved, lno );
}
//---------------------------------------------------------------------------------------
void CDDHLayer::setSaveLayerInfo( int saveind )
{
   if( saveind != 0 )
   {
      printf( "Using objects' Data element to store layer info\n" );
      mSaveLayerInfo = true;
   }
   else
   {
      printf( "Not using objects' Data element to store layer info\n" );
      mSaveLayerInfo = false;
   }
}
//---------------------------------------------------------------------------------------
bool CDDHLayer::getSaveLayerInfo()
{
   return mSaveLayerInfo;
}
//---------------------------------------------------------------------------------------
void CDDHLayer::removeUnusedObj()
{
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      CDDHData *d = mHead[ix];
      while( d != NULL )
      {
         ACObject *o = ( ACObject * )d->data;
         // If this object doesn't exist or is the WORLD object (object can
         // sometimes be a world object between unloading/loading of documents
         // as the world object takes the memory location of a previously
         // layered object)
         if( !object_exists( o ) || ac_object_get_type( o ) == OBJECT_WORLD )
         {
            CDDHData *dtmp = d;
            d = d->next;
            removeObjFromLayer( ix, dtmp );
         }
         else
         {
            d = d->next;
         }
      }
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::getObjLayersFromUserData( ACObject *o )
{
   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      ACObject *obj = ( ACObject * )p->data;
      // Assign based on layer data in user data fiels
      int i = getLayerFromObjData( obj );
      if( i > -1 && i < DDHLAYER_COUNT )
      {
         moveObjToLayer( i, obj );
      }
      getObjLayersFromUserData( obj );
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::writeLayerInfoToObjData()
{
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      CDDHData *d = mHead[ix];
      while( d != NULL )
      {
         ACObject *o = ( ACObject * )d->data;
         // If object no longer exists in AC3D, remove this reference
         if( !object_exists( o ) || ac_object_get_type( o ) == OBJECT_WORLD )
         {
            CDDHData *dtmp = d;
            d = d->next;
            removeObjFromLayer( ix, dtmp );
         }
         else
         {
            updateObjDataInfo( o, ix );
            d = d->next;
         }
      }
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::showLayer( int lno, Boolean ind )
{
   if( lno < 0 || lno >= DDHLAYER_COUNT )
   {
      return;
   }
   CDDHData *d = mHead[lno];
   bool firstTime = true;
   List *hideobjs = NULL;
   while( d != NULL )
   {
      ACObject *o = ( ACObject * )d->data;
      // If object no longer exists in AC3D, remove this reference
      if( !object_exists( o ) || ac_object_get_type( o ) == OBJECT_WORLD )
      {
         CDDHData *dtmp = d;
         d = d->next;
         removeObjFromLayer( lno, dtmp );
      }
      else
      {
         list_add_item_head( &hideobjs, o );
         d = d->next;
      }
   }
   if( hideobjs != NULL )
   {
      if( ind )
      {
         add_undoable( "Show layer", new DDHUndoableHide( hideobjs, ind ));
      }
      else
      {
         add_undoable( "Hide layer", new DDHUndoableHide( hideobjs, ind ));
      }
      list_free( &hideobjs );
   }
}
//---------------------------------------------------------------------------------------
void CDDHLayer::selectLayer( int lno, Boolean ind )
{
   if( lno < 0 || lno >= DDHLAYER_COUNT )
   {
      return;
   }
   CDDHData *d = mHead[lno];
   add_undoable_change_selection( "change selection" );
   List *selobjs = NULL;
   while( d != NULL )
   {
      ACObject *o = ( ACObject * )d->data;
      // If object no longer exists in AC3D, remove this reference
      if( !object_exists( o ) || ac_object_get_type( o ) == OBJECT_WORLD )
      {
         CDDHData *dtmp = d;
         d = d->next;
         removeObjFromLayer( lno, dtmp );
      }
      else
      {
         ACObject *o = ( ACObject * )d->data;
         // Select the object
         if( ind )
         {
            // Make the object visible/invisible
            object_set_visible( o, TRUE );
            list_add_item_head( &selobjs, o );
         }
         // Unselect the object
         else
         {
            ac_unselect_object( o );
         }
         d = d->next;
      }
   }
   if( selobjs != NULL )
   {
      ac_selection_select_objectlist( selobjs );
      list_free( &selobjs );
   }
}
//---------------------------------------------------------------------------------------
int CDDHLayer::getObjectCount( int lno )
{
   int i = 0;
   for( CDDHData *d = mHead[lno]; d != NULL; d = d->next )
   {
      ++i;
   }
   return i;
}
//---------------------------------------------------------------------------------------

