#ifndef cddhlayer_H
#define cddhlayer_H

#include <stdio.h>
#include "ac_plugin.h"

#define DDHLAYER_COUNT (10)
#define DDHLAYER_STARTVAL ((int)'0')

//---------------------------------------------------------------------------------------
class CDDHData
{
public:
   CDDHData *prev;
   CDDHData *next;
   void *data;
   CDDHData()
   {
      prev = NULL;
      next = NULL;
      data = NULL;
   }
};
//---------------------------------------------------------------------------------------
class CDDHLayer
{

private:

   //! List of objects
   CDDHData *mHead[DDHLAYER_COUNT];
   CDDHData *mTail[DDHLAYER_COUNT];

   //! Whether to save layer info in each object's data field
   bool mSaveLayerInfo;

   //! Singleton instance exists
   static bool mInstanceInd;
   //! Singleton instance
   static CDDHLayer *mSingle;

   //! Constructor (private for singleton class)
   CDDHLayer();

   //! Returns data element for object within a given layer (or NULL if not in a layer)
   CDDHData *findObjInLayer( int lno, void *o );
   //! Returns the layer number in which an object exists, or -1 if not in a layer
   int findObjInAllLayers( void *o );
   //! Removes an object from its layer list
   void removeObjFromLayer( int lno, CDDHData *d );

   //! Retrieve the layer number from an object's data field, or -1 if not found
   int getLayerFromObjData( ACObject *o );

   //! Moves an object to the specified layer
   void moveObjToLayer( int lno, void *o );

   //! Set the object's data field to include layer info
   void updateObjDataInfo( ACObject *o, int lno );

   //! Recursive form of moveSelection, for moving grouped data
   void moveSelectionRecurse( List *objs, int lno, int &objsmoved );

public:

   //! Destructor
   ~CDDHLayer();

   //! Get the layer singleton
   static CDDHLayer *getSingleton();

   //! Clear all items from the list
   void clear();

   //! Move the selected object(s) into the specified layer
   void moveSelection( int lno );

   //! Set the mSaveLayerInfo indicator
   void setSaveLayerInfo( int saveind );
   //! Get the mSaveLayerInfo indicator
   bool getSaveLayerInfo();

   //! Remove objects no longer in existence from lists
   void removeUnusedObj();
   //! Retrieve layer info from objects' data fields (includes children of ACObject parm)
   void getObjLayersFromUserData( ACObject *o );

   //! Write layer data to the objects' data fields
   void writeLayerInfoToObjData();

   //! Show the objects in the layer
   void showLayer( int lno, Boolean ind );
   //! Select the objects in the layer
   void selectLayer( int lno, Boolean ind );

   //! Return the number of objects in a layer
   int getObjectCount( int lno );

};
//---------------------------------------------------------------------------------------

#endif

