#include "ac_plugin.h"
#include "cddhlayer.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern "C"
{
typedef void (ACSelectionChangedCallback)(char *reason, void *userdata);
Prototype void ac_remove_selection_changed_callback(ACSelectionChangedCallback *c);
Prototype int ac_add_selection_changed_callback(ACSelectionChangedCallback *c, void *userdata);
}

// Whether changeselection callback was registered
static bool ddhlayer_g_changeCBRegistered = false;

// Number of objects recorded in the scene
static int ddhlayer_g_num_objects = 0;

//---------------------------------------------------------------------------------------
// Detects whether the number of objects has changed since the last time this function
// was called
static bool ddhlayer_numObjectsChanged()
{
   int nobs = ac_object_get_num_children( ac_get_world() );
   bool stat = ( nobs != ddhlayer_g_num_objects );
   ddhlayer_g_num_objects = nobs;
   return stat;
}
//---------------------------------------------------------------------------------------
static void ddhlayer_show( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->showLayer( atoi( argv[1] ), TRUE );
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_hide( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->showLayer( atoi( argv[1] ), FALSE );
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_move( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->moveSelection( atoi( argv[1] ));
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      //tcl_command( "set g_ddhlayer_count%d %d", ix, l->getObjectCount( ix ));
      tcl_command( ".layer.frm%d.txtobjs%d configure -text %d",
                    ix, ix, l->getObjectCount( ix ));
   }
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_usedata( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->setSaveLayerInfo( atoi( argv[1] ));
   if( l->getSaveLayerInfo() )
   {
      l->writeLayerInfoToObjData();
   }
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      //tcl_command( "set g_ddhlayer_count%d %d", ix, l->getObjectCount( ix ));
      tcl_command( ".layer.frm%d.txtobjs%d configure -text %d",
                    ix, ix, l->getObjectCount( ix ));
   }
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_refreshlayerlist()
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->removeUnusedObj();
   l->getObjLayersFromUserData( ac_get_world() );
   for( int ix = 0; ix < DDHLAYER_COUNT; ++ix )
   {
      //tcl_command( "set g_ddhlayer_count%d %d", ix, l->getObjectCount( ix ));
      tcl_command( ".layer.frm%d.txtobjs%d configure -text %d",
                    ix, ix, l->getObjectCount( ix ));
   }
}
//---------------------------------------------------------------------------------------
static void ddhlayer_select( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->selectLayer( atoi( argv[1] ), TRUE );
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_deselect( int argc, char *argv[] )
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   l->selectLayer( atoi( argv[1] ), FALSE );
   redraw_all();
   display_status();
}
//---------------------------------------------------------------------------------------
static void ddhlayer_cb_selection_changed( char *why, void *userdata )
{
   // Do a quick check to make sure we "think" we're registered
   // Can probably remove this with the unregister callback working - left in place
   // for now until testing can be done to remove the conditional.
   if( ddhlayer_g_changeCBRegistered && ddhlayer_numObjectsChanged() )
   {
      ddhlayer_refreshlayerlist();
   }
}
//---------------------------------------------------------------------------------------
static void ddhlayer_register_callback()
{
   if( !ddhlayer_g_changeCBRegistered )
   {
      printf( "Registering layer select-change callback\n" );
      ac_add_selection_changed_callback( ddhlayer_cb_selection_changed, NULL );
      ddhlayer_g_changeCBRegistered = true;
   }
}
//---------------------------------------------------------------------------------------
static void ddhlayer_unregister_callback()
{
   if( ddhlayer_g_changeCBRegistered )
   {
      // Unregister callback
      ac_remove_selection_changed_callback( ddhlayer_cb_selection_changed );
      printf( "Unregistering layer select-change callback\n" );
      ddhlayer_g_changeCBRegistered = false;
   }
}
//---------------------------------------------------------------------------------------
static void ddhlayer_clear()
{
   CDDHLayer *l = CDDHLayer::getSingleton();
   delete l;
   ddhlayer_unregister_callback();
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   if( ac_get_version_number() < 5.0 - 0.00005 ||
       ( ac_get_version_number() < 5.0 + 0.00005 &&
         ac_get_version_revision() < 20.99999 ))
   {
      message_dialog( "Sorry, Layers works only in AC3D 5.0.21 and later.  If you are a registered user of AC3D 4.0 or 5.0, please download version 5.0.21 from www.ac3d.org" );
      return 1;
   }

   static bool firstTime = true;
   if( firstTime )
   {

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/layer.tcl\"", path );
      ac_add_tools_menu_item( "Layers...", "layer_dialog", "Use layer interface" );
      // Using argv:
      // bool funcname(int argc, const char* argv[])
      // ac_add_command_full( "cmdname", funcname, <num_args>, "argv", "pdesc", "desc" );
      ac_add_command_full( "ddhlayer_show", ( void * )ddhlayer_show, 1, "argv", "<layer#>",
                           "Layer show" );
      ac_add_command_full( "ddhlayer_hide", ( void * )ddhlayer_hide, 1, "argv", "<layer#>",
                           "Layer hide" );
      ac_add_command_full( "ddhlayer_move", ( void * )ddhlayer_move, 1, "argv", "<layer#>",
                           "Layer move" );
      ac_add_command_full( "ddhlayer_usedata", ( void * )ddhlayer_usedata, 1, "argv", "<use_data_ind>",
                           "Layer move" );
      ac_add_command( "ddhlayer_refreshlayerlist", ddhlayer_refreshlayerlist );
      ac_add_command( "ddhlayer_clear", ddhlayer_clear );
      ac_add_command_full( "ddhlayer_select", ( void * )ddhlayer_select, 1, "argv", "<layer#>",
                           "Layer select" );
      ac_add_command_full( "ddhlayer_deselect", ( void * )ddhlayer_deselect, 1, "argv", "<layer#>",
                           "Layer deselect" );
      ac_add_command( "ddhlayer_register_callback", ddhlayer_register_callback );
      ac_add_command( "ddhlayer_unregister_callback", ddhlayer_unregister_callback );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   printf( "Deleting layer singleton\n" );
   CDDHLayer *l = CDDHLayer::getSingleton();
   delete l;
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Layers - version 1.0.2 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Layers" );
}
//---------------------------------------------------------------------------------------

